; PostGIS for PostgreSQL
;
; postgis.nsi - NSIS Script for NSIS 2.0
; Requires the ExecDos plugin (http://nsis.sourceforge.net/wiki/ExecDos)
;
; Copyright (c) 2003,2004 Steffen Macke, DORSCH Consult
; Copyright (c) 2004-2007 Mark Cave-Ayland
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


; Sections
!include "Sections.nsh"

; Modern User Interface
!include "MUI.nsh"

ReserveFile "${NSISDIR}\Plugins\InstallOptions.dll"
ReserveFile ".\postgis_dbconn.ini"
ReserveFile ".\postgis_dbname.ini"

; Constants

; Database connection info
!define UserName $7
!define Password $8
!define Port	 $9

; If creating a database after installation
!define DataBase $6

!define PG_VERSION "8.0"
!define GEOS_VERSION "3.0.0rc4"
!define PROJ_VERSION "4.5.0"

!define POSTGIS_VERSION "1.3.1"
!define POSTGIS_PRODUCT "PostGIS ${POSTGIS_VERSION} for PostgreSQL ${PG_VERSION}"
!define POSTGIS_PACKAGING "1"
!define POSTGIS_PG_VERSION "pg80"

; Use the new WIN32DIST build directory
!define WIN32DIST_DIR "..\win32dist"

; Place all temporary files used by the installer in their own subdirectory
; under $TEMP (makes the files easier to find)
!define TEMPDIR "$TEMP\postgis_installer"

; The name of the PostGIS template database
!define POSTGIS_TEMPLATE "template_postgis"


; --------------------------------
; Configuration

	; General
	Name "${POSTGIS_PRODUCT}"
	OutFile "postgis-${POSTGIS_PG_VERSION}-setup-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"

	; Folder selection page
	InstallDir "C:\Program Files\PostgreSQL\${PG_VERSION}\"
  
	; Remember install folder
	InstallDirRegKey HKCU "Software\${POSTGIS_PRODUCT}" ""

; --------------------------------
; Interface Settings

	!define MUI_ABORTWARNING
	!define MUI_COMPONENTSPAGE_CHECKBITMAP "${NSISDIR}\Contrib\Graphics\Checks\colorful.bmp"
	!define MUI_ICON "${NSISDIR}\Contrib\Graphics\Icons\box-install.ico"
	!define MUI_UNICON "${NSISDIR}\Contrib\Graphics\Icons\box-uninstall.ico"
	!define MUI_HEADERIMAGE
	!define MUI_HEADERIMAGE_BITMAP "elephant.bmp"
	!define MUI_HEADERIMAGE_UNBITMAP "elephant.bmp"
  
; --------------------------------
; Install Pages

	!insertmacro MUI_PAGE_LICENSE "COPYING"

	!insertmacro MUI_PAGE_COMPONENTS

	!insertmacro MUI_PAGE_DIRECTORY

	; Custom database connection page
	Page custom PageDBConnInit PageDBConnExit ": Database Connection"
	
	; Custom database name page (only displayed if the create database
	; option is selected)
	Page custom PageDBNameInit PageDBNameExit ": Database Name"

	!insertmacro MUI_PAGE_INSTFILES
  
; --------------------------------
; Uninstall Pages

	!insertmacro MUI_UNPAGE_CONFIRM
	!insertmacro MUI_UNPAGE_INSTFILES  

;--------------------------------
; Languages
 
	!insertmacro MUI_LANGUAGE "English"

;--------------------------------
; Installer Sections

Section "PostGIS" SecPostGIS

	; By setting the output path to the temporary directory, we force NSIS
	; to make the directory exist. This makes debugging the installer so
	; much easier! Note that the file list in inputfile.nsh overrides this
	; value anyway...
	SetOutPath "${TEMPDIR}"

	; This section must always be installed
	SectionIn RO

	; Set the shell context
	SetShellVarContext all
	
	; Checks whether installation folder is the PostgreSQL installation folder
	IfFileExists "$INSTDIR\bin\postgres.exe" postgres_available
	MessageBox MB_OK|MB_ICONSTOP "PostGIS has to be installed to the PostgreSQL folder."
	Abort "PostGIS has to be installed to the PostgreSQL folder."


	postgres_available:  

		;
		; Start the installation
		;

		; Read the database connection settings
		ReadINIStr ${UserName} "$PLUGINSDIR\postgis_dbconn.ini" "Field 3" "State"
		DetailPrint "UserName=${UserName}"
		ReadINIStr ${Password} "$PLUGINSDIR\postgis_dbconn.ini" "Field 5" "State"
		DetailPrint "Password=${Password}"
		ReadINIStr ${Port} "$PLUGINSDIR\postgis_dbconn.ini" "Field 7" "State"
		DetailPrint "Port=${Port}"


		; Check to see whether the template database "template_postgis" exists
		Call CheckForTemplatePostGIS
		IntCmp $R9 1 postgis_template_exists postgis_install

		postgis_template_exists:
			; The template_postgis database exists from a previous installation
			MessageBox MB_YESNO "WARNING: The installer has detected that the PostGIS template database '${POSTGIS_TEMPLATE}' already exists in your PostgreSQL installation. In order to install this version of PostGIS, it is necessary to create a new '${POSTGIS_TEMPLATE}' database, so any manual changes you have made to this database will be lost.$\r$\n$\r$\nDo you still wish to proceed with the installation?" IDYES postgis_remove_template IDNO postgis_abort_install

			postgis_remove_template:
				; We need to remove the existing template_postgis
				Call RemoveTemplatePostGIS
				Goto postgis_install

			postgis_abort_install:
				; Abort the installation
				Abort

		postgis_install:

				; Include the input files
				!include inputfiles.nsh
				
				; Create the template database
				Call CreateTemplatePostGIS


		; Create documentation shortcuts for PostGIS documentation and JDBC documentation
		CreateDirectory "$SMPROGRAMS\${POSTGIS_PRODUCT}"
		CreateShortcut "$SMPROGRAMS\${POSTGIS_PRODUCT}\PostGIS ${POSTGIS_VERSION} Documentation.lnk" "$INSTDIR\doc\contrib\html\postgis\postgis.html"
		CreateShortcut "$SMPROGRAMS\${POSTGIS_PRODUCT}\PostGIS ${POSTGIS_VERSION} JDBC Driver Documentation.lnk" "$INSTDIR\doc\contrib\html\postgis\jdbc\index.html"

		; Store install folder
		WriteRegStr HKCU "Software\${POSTGIS_PRODUCT}" "" $INSTDIR

		WriteRegStr HKEY_LOCAL_MACHINE "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "DisplayName" "${POSTGIS_PRODUCT} (remove only)"
		WriteRegStr HKEY_LOCAL_MACHINE "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "UninstallString" '"$INSTDIR\uninstall-postgis-${POSTGIS_PG_VERSION}-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"'
		WriteRegStr HKEY_LOCAL_MACHINE "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "DisplayIcon" '"$INSTDIR\uninstall-postgis-${POSTGIS_PG_VERSION}-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"'

		; Create uninstaller
		WriteUninstaller "$INSTDIR\uninstall-postgis-${POSTGIS_PG_VERSION}-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"

SectionEnd


Section "Create spatial database" SecCreateDb

		; Read the database name
		ReadINIStr ${DataBase} "$PLUGINSDIR\postgis_dbname.ini" "Field 3" "State"
		DetailPrint "DataBase=${DataBase}"

		; Create the database
		Call CreateSpatialDatabase
     
SectionEnd


;--------------------------------
; Language Strings

LangString DESC_SecPostGIS ${LANG_ENGLISH} "Install PostGIS ${POSTGIS_VERSION} for PostgreSQL ${PG_VERSION}, GEOS ${GEOS_VERSION}, PROJ ${PROJ_VERSION}."
LangString DESC_SecCreateDb ${LANG_ENGLISH} "Create a new spatial database."

!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
	!insertmacro MUI_DESCRIPTION_TEXT ${SecPostGIS} $(DESC_SecPostGIS)
	!insertmacro MUI_DESCRIPTION_TEXT ${SecCreateDb} $(DESC_SecCreateDb)
!insertmacro MUI_FUNCTION_DESCRIPTION_END

 
;--------------------------------
; Uninstaller Section

Section "Uninstall"

	;
	; Uninstall files - as yet we do not try to remove the template_postgis database
	;

	!include uninstallfiles.nsh

	; Delete the directory itself
	RmDir  "$INSTDIR\share\contrib\postgis"

	; Remove documentation shortcuts
	SetShellVarContext all
	RmDir /r "$SMPROGRAMS\${POSTGIS_PRODUCT}"

	; Remove installer registry entries
	DeleteRegKey /ifempty HKCU "Software\${POSTGIS_PRODUCT}"
	DeleteRegValue HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "DisplayName"
	DeleteRegValue HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "UninstallString"
	DeleteRegValue HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}" "DisplayIcon"
	DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${POSTGIS_PRODUCT}"

	; Remove the uninstaller itself
	Delete "$INSTDIR\uninstall-postgis-${POSTGIS_PG_VERSION}-${POSTGIS_VERSION}-${POSTGIS_PACKAGING}.exe"
  
SectionEnd


;
; Miscellaneous functions
;

Function .onInit

	; Initialise the plugins dir
	InitPluginsDir

	; Extract the database connection page info
	File /oname=$PLUGINSDIR\postgis_dbconn.ini ".\postgis_dbconn.ini"

	; Extract the database name page info
	File /oname=$PLUGINSDIR\postgis_dbname.ini ".\postgis_dbname.ini"

FunctionEnd


;
; Init/Exit functions for the database connection page
;

Function PageDBConnInit

	; Set the header text
	!insertmacro MUI_HEADER_TEXT "Database Connection" "Specify the database connection"

	; Display the InstallOptions dialog
	Push $R0
	InstallOptions::dialog $PLUGINSDIR\postgis_dbconn.ini
	Pop $R0

FunctionEnd


Function PageDBConnExit
	; Do nothing
FunctionEnd


;
; Init/Exit functions for the database name page
;

Function PageDBNameInit

	; Get information from the "Create Database" item on the components page
	SectionGetFlags ${SecCreateDB} $R9

	; If the item is not selected ($R9 == 0) then skip this page
	IntCmp $R9 0 pdi_skip pdi_noskip

	pdi_noskip:
		; Create database option selected - show this page

		; Set the header text
		!insertmacro MUI_HEADER_TEXT "Database Name" "Specify the name of the spatial database to be created at the end of the installation process"

		; Display the InstallOptions dialog
		Push $R0
		InstallOptions::dialog $PLUGINSDIR\postgis_dbname.ini
		Pop $R0

		; Exit function
		Return

	pdi_skip:
		; Create database option not selected - skip this page
		Abort

FunctionEnd


Function PageDBNameExit
	; Do nothing
FunctionEnd



;
; Main installer functions
;

;
; CheckForTemplatePostGIS - determine whether the template_postgis
; database exists within the cluster. Returns 0 (False) or 1 (True)
; in R9.
;

Function CheckForTemplatePostGIS

	; File "check_templatepostgis.bat"
	FileOpen $1 "${TEMPDIR}\check_templatepostgis.bat" "w"
	FileWrite $1 "set PGPASSWORD=${Password}$\r$\n"
	FileWrite $1 "set PGPORT=${Port}$\r$\n"
	FileWrite $1 '"$INSTDIR\bin\psql.exe" -U ${UserName} -h localhost -l -t -c "SELECT $\'true$\' FROM pg_database WHERE datname=$\'${POSTGIS_TEMPLATE}$\'" -d template1 > "${TEMPDIR}\check_templatepostgis_output.txt" 2> "${TEMPDIR}\check_templatepostgis_error.txt"$\r$\n'
	FileClose $1

	; Execute check_templatepostgis.bat
	DetailPrint "Checking for existing PostGIS installation..."
	ExecDos::exec '"$SYSDIR\cmd.exe" /c "${TEMPDIR}\check_templatepostgis.bat"'

	; If the string "true" is found once in the output then the template_postgis db exists
	StrCpy $R1 "true"
	StrCpy $R0 "${TEMPDIR}\check_templatepostgis_output.txt"
	Call SearchFileForString
	IntCmp $R9 1 cftp_template_exists

	; Template doesn't exist - return 0
	StrCpy $R9 0
	Return

	cftp_template_exists:
		; Template exists - return 1
		StrCpy $R9 1
		Return

FunctionEnd


;
; RemoveTemplatePostGIS - removes the template_postgis db
;

Function RemoveTemplatePostGIS

	; File "remove_templatepostgis.bat"
	FileOpen $1 "${TEMPDIR}\remove_templatepostgis.bat" "w"
	FileWrite $1 "set PGPASSWORD=${Password}$\r$\n"
	FileWrite $1 "set PGPORT=${Port}$\r$\n"

	; NB: First we need to remove the datistemplate flag before we can drop it
	FileWrite $1 '"$INSTDIR\bin\psql.exe" -U ${UserName} -h localhost -c "UPDATE pg_database SET datistemplate=$\'f$\' WHERE datname=$\'${POSTGIS_TEMPLATE}$\'" ${POSTGIS_TEMPLATE} > "${TEMPDIR}\remove_templatepostgis_output.txt" 2> "${TEMPDIR}\remove_templatepostgis_error.txt"$\r$\n'

	; Drop the database
	FileWrite $1 '"$INSTDIR\bin\dropdb.exe" -U ${UserName} -h localhost ${POSTGIS_TEMPLATE} >> "${TEMPDIR}\remove_templatepostgis_output.txt" 2>> "${TEMPDIR}\remove_templatepostgis_error.txt"$\r$\n'
	FileClose $1

	; Execute check_templatepostgis.bat
	DetailPrint "Removing existing ${POSTGIS_TEMPLATE} database..."
	ExecDos::exec '"$SYSDIR\cmd.exe" /c "${TEMPDIR}\remove_templatepostgis.bat"'

	; If the string "DROP DATABASE" is returned then everything went as planned
	StrCpy $R1 "DROP DATABASE"
	StrCpy $R0 "${TEMPDIR}\remove_templatepostgis_output.txt"
	Call SearchFileForString
	IntCmp $R9 1 rtp_removalsuccess

	; Looks like the removal failed so error out
	StrCpy $R8 "Removal of ${POSTGIS_TEMPLATE} database failed."
	StrCpy $R9 "${TEMPDIR}\remove_templatepostgis_error.txt"
	Call AbortDisplayLogOption

	rtp_removalsuccess:
		; Return to caller
		Return

FunctionEnd


;
; CreateTemplatePostGIS - create the template_postgis db by loading in the lwpostgis.sql
; and spatial_ref_sys.sql files.
;

Function CreateTemplatePostGIS

	; File "create_templatepostgis_db.bat"
	FileOpen $1 "${TEMPDIR}\create_templatepostgis_db.bat" "w"
	FileWrite $1 "set PGPASSWORD=${Password}$\r$\n"
	FileWrite $1 "set PGPORT=${Port}$\r$\n"
	FileWrite $1 '"$INSTDIR\bin\createdb.exe" -U ${UserName} -h localhost --encoding=UNICODE ${POSTGIS_TEMPLATE} > "${TEMPDIR}\create_templatepostgis_db_output.txt" 2> "${TEMPDIR}\create_templatepostgis_db_error.txt"$\r$\n'
	FileClose $1

	; Execute create_postgis_db.bat
	DetailPrint "Creating template database..."
	ExecDos::exec '"$SYSDIR\cmd.exe" /c "${TEMPDIR}\create_templatepostgis_db.bat"'

	; If the string "CREATE DATABASE" is returned then everything went as planned
	StrCpy $R1 "CREATE DATABASE"
	StrCpy $R0 "${TEMPDIR}\create_templatepostgis_db_output.txt"
	Call SearchFileForString
	IntCmp $R9 1 ctp_createdb_success

	; Looks like the creation failed so we abort
	StrCpy $R8 "Database creation failed."
	StrCpy $R9 "${TEMPDIR}\create_templatepostgis_db_error.txt"
	Call AbortDisplayLogOption

	; Creation went well, so lets continue
	ctp_createdb_success:

		; File "create_templatepostgis_lang.bat"
		FileOpen $1 "${TEMPDIR}\create_templatepostgis_lang.bat" "w"
		FileWrite $1 "set PGPASSWORD=${Password}$\r$\n"
		FileWrite $1 "set PGPORT=${Port}$\r$\n"
		FileWrite $1 '"$INSTDIR\bin\createlang.exe" -U ${UserName} -h localhost plpgsql ${POSTGIS_TEMPLATE} > "${TEMPDIR}\create_templatepostgis_lang_output.txt" 2> "${TEMPDIR}\create_templatepostgis_lang_error.txt"$\r$\n'

		; NB: We also set the template flag in the pg_database table
		FileWrite $1 '"$INSTDIR\bin\psql.exe" -U ${UserName} -h localhost -c "UPDATE pg_database SET datistemplate=$\'t$\' WHERE datname=$\'${POSTGIS_TEMPLATE}$\'" ${POSTGIS_TEMPLATE} > "${TEMPDIR}\create_templatepostgis_lang_output.txt" 2> "${TEMPDIR}\create_templatepostgis_lang_error.txt"$\r$\n'
		FileClose $1

		; Execute create_postgis_lang.bat
		DetailPrint "Loading PLPGSQL language..."
		ExecDos::exec '"$SYSDIR\cmd.exe" /c "${TEMPDIR}\create_templatepostgis_lang.bat"'

		; Now createlang is different to the others. If the error output has a length > 0 then
		; we assume the command failed.
		FileOpen $1 "${TEMPDIR}\create_templatepostgis_lang_error.txt" "r"
		FileRead $1 $2
		FileClose $1
		StrLen $1 $2
		IntCmp $1 0 ctp_createlang_succeeded

		; An error occurred so abort the installation
		StrCpy $R8 "Database installation of PLPGSQL failed."
		StrCpy $R9 "${TEMPDIR}\create_templatepostgis_lang_error.txt"
		Call AbortDisplayLogOption


	; PLPGSQL has been successfully loaded into the database, so continue
	ctp_createlang_succeeded:

		; File "create_templatepostgis_sql.bat"
		FileOpen $1 "${TEMPDIR}\create_templatepostgis_sql.bat" "w"
		FileWrite $1 "set PGPASSWORD=${Password}$\r$\n"
		FileWrite $1 "set PGPORT=${Port}$\r$\n"
		FileWrite $1 '"$INSTDIR\bin\psql.exe" -U ${UserName} -h localhost -f "$INSTDIR\share\contrib\lwpostgis.sql" ${POSTGIS_TEMPLATE} > "${TEMPDIR}\create_templatepostgis_sql_output.txt" 2> "${TEMPDIR}\create_templatepostgis_sql_error.txt"$\r$\n'
		FileWrite $1 '"$INSTDIR\bin\psql.exe" -U ${UserName} -h localhost -f "$INSTDIR\share\contrib\spatial_ref_sys.sql" ${POSTGIS_TEMPLATE} >> "${TEMPDIR}\create_templatepostgis_sql_output.txt" 2>> "${TEMPDIR}\create_templatepostgis_sql_error.txt"$\r$\n'
		FileClose $1

		; Execute create_postgis_sql.bat
		DetailPrint "Loading PostGIS functions..."
		ExecDos::exec '"$SYSDIR\cmd.exe" /c "${TEMPDIR}\create_templatepostgis_sql.bat"'

		; If the string "ERROR" is found on the error output then the install failed
		StrCpy $R1 "ERROR"
		StrCpy $R0 "${TEMPDIR}\create_templatepostgis_sql_error.txt"
		Call SearchFileForString
		IntCmp $R9 0 ctp_loadsql_success

		; If we get here then the load failed - abort installation
		StrCpy $R8 "Database installation of PostGIS functions failed ($R9 errors detected)."
		StrCpy $R9 "${TEMPDIR}\create_templatepostgis_sql_error.txt"
		Call AbortDisplayLogOption
		

	ctp_loadsql_success:
		; We have successfully created the template_postgis database
		Return

FunctionEnd


;
; CreateSpatialDatabase - create a spatial database by copying from template_postgis
;

Function CreateSpatialDatabase

	; File "create_spatial_db.bat"
	FileOpen $1 "${TEMPDIR}\create_spatial_db.bat" "w"
	FileWrite $1 "set PGPASSWORD=${Password}$\r$\n"
	FileWrite $1 "set PGPORT=${Port}$\r$\n"
	FileWrite $1 '"$INSTDIR\bin\createdb.exe" -U ${UserName} -h localhost -T ${POSTGIS_TEMPLATE} ${DataBase} > "${TEMPDIR}\create_spatial_db_output.txt" 2> "${TEMPDIR}\create_spatial_db_error.txt"$\r$\n'
	FileClose $1

	; Execute create_spatial_db.bat
	DetailPrint "Creating spatial database..."
	ExecDos::exec '"$SYSDIR\cmd.exe" /c "${TEMPDIR}\create_spatial_db.bat"'

	; If the string "CREATE DATABASE" is returned then everything went as planned
	StrCpy $R1 "CREATE DATABASE"
	StrCpy $R0 "${TEMPDIR}\create_spatial_db_output.txt"
	Call SearchFileForString
	IntCmp $R9 1 csd_createdb_success

	; Looks like the creation failed so we abort
	StrCpy $R8 "Spatial database creation failed."
	StrCpy $R9 "${TEMPDIR}\create_spatial_db_error.txt"
	Call AbortDisplayLogOption

	csd_createdb_success:
		; Simply return
		Return

FunctionEnd


;
; SearchFileForString - search through the file for the given string
; returning the number of occurances in R9
;
;	R0 - filename
;	R1 - string
;

Function SearchFileForString

	; Save all normal registers
	Push $0
	Push $1
	Push $2
	Push $3
	Push $4
	Push $5
	Push $6
	Push $7
	Push $8
	Push $9

	; Open the file for reading
	ClearErrors
	FileOpen $0 $R0 "r"
	IfErrors sfs_openerror

	; Get the length of the string in $1 and the current total in $2
	StrLen $1 $R1
	StrCpy $2 0

	sfs_lineloop:

		; Read in the next line from the file into $3
		FileRead $0 $3
		IfErrors sfs_done

		; Store the current position in $4
		StrCpy $4 0

		; Store the length of the line read from the file in $5
		StrLen $5 $3

		sfs_charloop:
		
			; Copy the string from the current position with the length of
			; the string we are searching for into $6
			StrCpy $6 $3 $1 $4
			
			; If the string in $6 matches the string we passed in R1 then we
			; have a match, so increment the total
			StrCmpS $R1 $6 sfs_foundstr sfs_notfoundstr

			sfs_foundstr:
				; We found a match so increment the total in $2
				IntOp $2 $2 + 1

			sfs_notfoundstr:
				; No match so increment our current position
				IntOp $4 $4 + 1

				; If we have reached the end of the string in $3 then exit,
				; otherwise repeat sfs_charloop
				IntCmp $4 $5 sfs_charloop sfs_charloop sfs_endcharloop

		sfs_endcharloop:
				; Finished this line so read the next line
				Goto sfs_lineloop

		sfs_done:
				; Finished reading the file - total is in $2 but we want to
				; return it in R9
				StrCpy $R9 $2
		
				; Restore all normal registers from the stack
				Pop $9
				Pop $8
				Pop $7
				Pop $6
				Pop $5
				Pop $4
				Pop $3
				Pop $2
				Pop $1
				Pop $0

				; End of function
				Return

		sfs_openerror:
				; Couldn't open the specified file so abort
				Abort "ERROR: Could not open file $R0"
				Return

FunctionEnd


;
; AbortDisplayLogOption - give the user the option to display an error log
; and abort the installation.
;
;	R8 - Error message
;	R9 - Log filename
;

Function AbortDisplayLogOption

	; Display a message box with the error
	MessageBox MB_YESNO|MB_ICONSTOP "$R8$\r$\n$\r$\nWould you like to view the error log '$R9'?" IDYES adlo_show_error_log 
	
	; If the user selects NO, simply abort the installation
	DetailPrint "$R8"
	Abort "$R8"
	Return
	
	adlo_show_error_log:
		; Otherwise show the error log first
		ExecShell "open" "$R9"
		DetailPrint "$R8"
		Abort "$R8"
		Return

FunctionEnd
